@@ -134,7 +134,7 @@ objects from the manifest much like DLO. If any of the segments from the
 manifest are not found or their Etag/Content Length have changed since upload,
 the connection will drop. In this case a 409 Conflict will be logged in the
 proxy logs and the user will receive incomplete results. Note that this will be
-enforced regardless of whether the user perfomed per-segment validation during
+enforced regardless of whether the user performed per-segment validation during
 upload.
 
 The headers from this GET or HEAD request will return the metadata attached
@@ -202,6 +202,7 @@ metadata which can be used for stats purposes.
 
 from six.moves import range
 
+from collections import defaultdict
 from datetime import datetime
 import json
 import mimetypes
@@ -213,12 +214,11 @@ from swift.common.exceptions import ListingIterError, SegmentError
 from swift.common.swob import Request, HTTPBadRequest, HTTPServerError, \
     HTTPMethodNotAllowed, HTTPRequestEntityTooLarge, HTTPLengthRequired, \
     HTTPOk, HTTPPreconditionFailed, HTTPException, HTTPNotFound, \
-    HTTPUnauthorized, HTTPConflict, HTTPRequestedRangeNotSatisfiable,\
-    Response, Range
+    HTTPUnauthorized, HTTPConflict, Response, Range
 from swift.common.utils import get_logger, config_true_value, \
     get_valid_utf8_str, override_bytes_from_content_type, split_path, \
     register_swift_info, RateLimitedIterator, quote, close_if_possible, \
-    closing_if_possible
+    closing_if_possible, LRUCache, StreamingPile
 from swift.common.request_helpers import SegmentedIterable
 from swift.common.constraints import check_utf8, MAX_BUFFERED_SLO_SEGMENTS
 from swift.common.http import HTTP_NOT_FOUND, HTTP_UNAUTHORIZED, is_success
@@ -297,11 +297,11 @@ def parse_and_validate_input(req_body, req_path):
                              for ek in sorted(extraneous_keys))))
             continue
 
-        if not isinstance(seg_dict['path'], basestring):
+        if not isinstance(seg_dict['path'], six.string_types):
             errors.append("Index %d: \"path\" must be a string" % seg_index)
             continue
         if not (seg_dict['etag'] is None or
-                isinstance(seg_dict['etag'], basestring)):
+                isinstance(seg_dict['etag'], six.string_types)):
             errors.append(
                 "Index %d: \"etag\" must be a string or null" % seg_index)
             continue
@@ -386,8 +386,6 @@ class SloGetContext(WSGIContext):
 
     def __init__(self, slo):
         self.slo = slo
-        self.first_byte = None
-        self.last_byte = None
         super(SloGetContext, self).__init__(slo.app)
 
     def _fetch_sub_slo_segments(self, req, version, acc, con, obj):
@@ -434,7 +432,7 @@ class SloGetContext(WSGIContext):
             return int(seg_dict['bytes'])
 
     def _segment_listing_iterator(self, req, version, account, segments,
-                                  recursion_depth=1):
+                                  byteranges):
         for seg_dict in segments:
             if config_true_value(seg_dict.get('sub_slo')):
                 override_bytes_from_content_type(seg_dict,
@@ -448,23 +446,46 @@ class SloGetContext(WSGIContext):
         # If we were to make SegmentedIterable handle all the range
         # calculations, we would be unable to make this optimization.
         total_length = sum(self._segment_length(seg) for seg in segments)
-        if self.first_byte is None:
-            self.first_byte = 0
-        if self.last_byte is None:
-            self.last_byte = total_length - 1
+        if not byteranges:
+            byteranges = [(0, total_length - 1)]
 
+        # Cache segments from sub-SLOs in case more than one byterange
+        # includes data from a particular sub-SLO. We only cache a few sets
+        # of segments so that a malicious user cannot build a giant SLO tree
+        # and then GET it to run the proxy out of memory.
+        #
+        # LRUCache is a little awkward to use this way, but it beats doing
+        # things manually.
+        #
+        # 20 is sort of an arbitrary choice; it's twice our max recursion
+        # depth, so we know this won't expand memory requirements by too
+        # much.
+        cached_fetch_sub_slo_segments = \
+            LRUCache(maxsize=20)(self._fetch_sub_slo_segments)
+
+        for first_byte, last_byte in byteranges:
+            byterange_listing_iter = self._byterange_listing_iterator(
+                req, version, account, segments, first_byte, last_byte,
+                cached_fetch_sub_slo_segments)
+            for seg_info in byterange_listing_iter:
+                yield seg_info
+
+    def _byterange_listing_iterator(self, req, version, account, segments,
+                                    first_byte, last_byte,
+                                    cached_fetch_sub_slo_segments,
+                                    recursion_depth=1):
         last_sub_path = None
         for seg_dict in segments:
             seg_length = self._segment_length(seg_dict)
-            if self.first_byte >= seg_length:
+            if first_byte >= seg_length:
                 # don't need any bytes from this segment
-                self.first_byte -= seg_length
-                self.last_byte -= seg_length
+                first_byte -= seg_length
+                last_byte -= seg_length
                 continue
 
-            if self.last_byte < 0:
+            if last_byte < 0:
                 # no bytes are needed from this or any future segment
-                break
+                return
 
             seg_range = seg_dict.get('range')
             if seg_range is None:
@@ -483,33 +504,30 @@ class SloGetContext(WSGIContext):
                 sub_path = get_valid_utf8_str(seg_dict['name'])
                 sub_cont, sub_obj = split_path(sub_path, 2, 2, True)
                 if last_sub_path != sub_path:
-                    sub_segments = self._fetch_sub_slo_segments(
+                    sub_segments = cached_fetch_sub_slo_segments(
                         req, version, account, sub_cont, sub_obj)
                 last_sub_path = sub_path
 
                 # Use the existing machinery to slice into the sub-SLO.
-                # This requires that we save off our current state, and
-                # restore at the other end.
-                orig_start, orig_end = self.first_byte, self.last_byte
-                self.first_byte = range_start + max(0, self.first_byte)
-                self.last_byte = min(range_end, range_start + self.last_byte)
-
-                for sub_seg_dict, sb, eb in self._segment_listing_iterator(
+                for sub_seg_dict, sb, eb in self._byterange_listing_iterator(
                         req, version, account, sub_segments,
+                        # This adjusts first_byte and last_byte to be
+                        # relative to the sub-SLO.
+                        range_start + max(0, first_byte),
+                        min(range_end, range_start + last_byte),
+
+                        cached_fetch_sub_slo_segments,
                         recursion_depth=recursion_depth + 1):
                     yield sub_seg_dict, sb, eb
-
-                # Restore the first/last state
-                self.first_byte, self.last_byte = orig_start, orig_end
             else:
                 if isinstance(seg_dict['name'], six.text_type):
                     seg_dict['name'] = seg_dict['name'].encode("utf-8")
                 yield (seg_dict,
-                       max(0, self.first_byte) + range_start,
-                       min(range_end, range_start + self.last_byte))
+                       max(0, first_byte) + range_start,
+                       min(range_end, range_start + last_byte))
 
-            self.first_byte -= seg_length
-            self.last_byte -= seg_length
+            first_byte -= seg_length
+            last_byte -= seg_length
 
     def _need_to_refetch_manifest(self, req):
         """
@@ -692,22 +710,18 @@ class SloGetContext(WSGIContext):
 
     def _manifest_get_response(self, req, content_length, response_headers,
                                segments):
-        self.first_byte, self.last_byte = None, None
         if req.range:
-            byteranges = req.range.ranges_for_length(content_length)
-            if len(byteranges) == 0:
-                return HTTPRequestedRangeNotSatisfiable(request=req)
-            elif len(byteranges) == 1:
-                self.first_byte, self.last_byte = byteranges[0]
+            byteranges = [
                 # For some reason, swob.Range.ranges_for_length adds 1 to the
                 # last byte's position.
-                self.last_byte -= 1
-            else:
-                req.range = None
+                (start, end - 1) for start, end
+                in req.range.ranges_for_length(content_length)]
+        else:
+            byteranges = []
 
         ver, account, _junk = req.split_path(3, 3, rest_with_last=True)
         plain_listing_iter = self._segment_listing_iterator(
-            req, ver, account, segments)
+            req, ver, account, segments, byteranges)
 
         def is_small_segment((seg_dict, start_byte, end_byte)):
             start = 0 if start_byte is None else start_byte
@@ -785,7 +799,10 @@ class StaticLargeObject(object):
             'rate_limit_after_segment', '10'))
         self.rate_limit_segments_per_sec = int(self.conf.get(
             'rate_limit_segments_per_sec', '1'))
-        delete_concurrency = int(self.conf.get('delete_concurrency', '2'))
+        self.concurrency = min(1000, max(0, int(self.conf.get(
+            'concurrency', '2'))))
+        delete_concurrency = int(self.conf.get(
+            'delete_concurrency', self.concurrency))
         self.bulk_deleter = Bulk(
             app, {}, delete_concurrency=delete_concurrency, logger=self.logger)
 
@@ -838,93 +855,103 @@ class StaticLargeObject(object):
         if not out_content_type:
             out_content_type = 'text/plain'
         data_for_storage = []
-        slo_etag = md5()
-        last_obj_path = None
+        path2indices = defaultdict(list)
         for index, seg_dict in enumerate(parsed_data):
-            obj_name = seg_dict['path']
-            if isinstance(obj_name, six.text_type):
-                obj_name = obj_name.encode('utf-8')
-            obj_path = '/'.join(['', vrs, account, obj_name.lstrip('/')])
+            path2indices[seg_dict['path']].append(index)
 
-            if obj_path != last_obj_path:
-                last_obj_path = obj_path
-                sub_req = make_subrequest(
-                    req.environ, path=obj_path + '?',  # kill the query string
-                    method='HEAD',
-                    headers={'x-auth-token': req.headers.get('x-auth-token')},
-                    agent='%(orig)s SLO MultipartPUT', swift_source='SLO')
-                head_seg_resp = sub_req.get_response(self)
+        def do_head(obj_name):
+            obj_path = '/'.join(['', vrs, account,
+                                 get_valid_utf8_str(obj_name).lstrip('/')])
 
-            if head_seg_resp.is_success:
-                segment_length = head_seg_resp.content_length
-                if seg_dict.get('range'):
-                    # Since we now know the length, we can normalize the
-                    # range. We know that there is exactly one range
-                    # requested since we checked that earlier in
-                    # parse_and_validate_input().
-                    ranges = seg_dict['range'].ranges_for_length(
-                        head_seg_resp.content_length)
+            sub_req = make_subrequest(
+                req.environ, path=obj_path + '?',  # kill the query string
+                method='HEAD',
+                headers={'x-auth-token': req.headers.get('x-auth-token')},
+                agent='%(orig)s SLO MultipartPUT', swift_source='SLO')
+            return obj_name, sub_req.get_response(self)
 
-                    if not ranges:
-                        problem_segments.append([quote(obj_name),
-                                                 'Unsatisfiable Range'])
-                    elif ranges == [(0, head_seg_resp.content_length)]:
-                        # Just one range, and it exactly matches the object.
-                        # Why'd we do this again?
-                        del seg_dict['range']
-                        segment_length = head_seg_resp.content_length
-                    else:
-                        rng = ranges[0]
-                        seg_dict['range'] = '%d-%d' % (rng[0], rng[1] - 1)
-                        segment_length = rng[1] - rng[0]
-
-                if segment_length < 1:
-                    problem_segments.append(
-                        [quote(obj_name),
-                         'Too small; each segment must be at least 1 byte.'])
-                total_size += segment_length
-                if seg_dict['size_bytes'] is not None and \
-                        seg_dict['size_bytes'] != head_seg_resp.content_length:
-                    problem_segments.append([quote(obj_name), 'Size Mismatch'])
-                if seg_dict['etag'] is None or \
-                        seg_dict['etag'] == head_seg_resp.etag:
-                    if seg_dict.get('range'):
-                        slo_etag.update('%s:%s;' % (head_seg_resp.etag,
-                                                    seg_dict['range']))
-                    else:
-                        slo_etag.update(head_seg_resp.etag)
-                else:
-                    problem_segments.append([quote(obj_name), 'Etag Mismatch'])
-                if head_seg_resp.last_modified:
-                    last_modified = head_seg_resp.last_modified
-                else:
-                    # shouldn't happen
-                    last_modified = datetime.now()
-
-                last_modified_formatted = \
-                    last_modified.strftime('%Y-%m-%dT%H:%M:%S.%f')
-                seg_data = {'name': '/' + seg_dict['path'].lstrip('/'),
-                            'bytes': head_seg_resp.content_length,
-                            'hash': head_seg_resp.etag,
-                            'content_type': head_seg_resp.content_type,
-                            'last_modified': last_modified_formatted}
-                if seg_dict.get('range'):
-                    seg_data['range'] = seg_dict['range']
-
-                if config_true_value(
-                        head_seg_resp.headers.get('X-Static-Large-Object')):
-                    seg_data['sub_slo'] = True
-                data_for_storage.append(seg_data)
-
-            else:
+        def validate_seg_dict(seg_dict, head_seg_resp):
+            if not head_seg_resp.is_success:
                 problem_segments.append([quote(obj_name),
                                          head_seg_resp.status])
+                return 0, None
+
+            segment_length = head_seg_resp.content_length
+            if seg_dict.get('range'):
+                # Since we now know the length, we can normalize the
+                # range. We know that there is exactly one range
+                # requested since we checked that earlier in
+                # parse_and_validate_input().
+                ranges = seg_dict['range'].ranges_for_length(
+                    head_seg_resp.content_length)
+
+                if not ranges:
+                    problem_segments.append([quote(obj_name),
+                                             'Unsatisfiable Range'])
+                elif ranges == [(0, head_seg_resp.content_length)]:
+                    # Just one range, and it exactly matches the object.
+                    # Why'd we do this again?
+                    del seg_dict['range']
+                    segment_length = head_seg_resp.content_length
+                else:
+                    rng = ranges[0]
+                    seg_dict['range'] = '%d-%d' % (rng[0], rng[1] - 1)
+                    segment_length = rng[1] - rng[0]
+
+            if segment_length < 1:
+                problem_segments.append(
+                    [quote(obj_name),
+                     'Too small; each segment must be at least 1 byte.'])
+            if seg_dict['size_bytes'] is not None and \
+                    seg_dict['size_bytes'] != head_seg_resp.content_length:
+                problem_segments.append([quote(obj_name), 'Size Mismatch'])
+            if seg_dict['etag'] is not None and \
+                    seg_dict['etag'] != head_seg_resp.etag:
+                problem_segments.append([quote(obj_name), 'Etag Mismatch'])
+            if head_seg_resp.last_modified:
+                last_modified = head_seg_resp.last_modified
+            else:
+                # shouldn't happen
+                last_modified = datetime.now()
+
+            last_modified_formatted = \
+                last_modified.strftime('%Y-%m-%dT%H:%M:%S.%f')
+            seg_data = {'name': '/' + seg_dict['path'].lstrip('/'),
+                        'bytes': head_seg_resp.content_length,
+                        'hash': head_seg_resp.etag,
+                        'content_type': head_seg_resp.content_type,
+                        'last_modified': last_modified_formatted}
+            if seg_dict.get('range'):
+                seg_data['range'] = seg_dict['range']
+            if config_true_value(
+                    head_seg_resp.headers.get('X-Static-Large-Object')):
+                seg_data['sub_slo'] = True
+            return segment_length, seg_data
+
+        data_for_storage = [None] * len(parsed_data)
+        with StreamingPile(self.concurrency) as pile:
+            for obj_name, resp in pile.asyncstarmap(do_head, (
+                    (path, ) for path in path2indices)):
+                for i in path2indices[obj_name]:
+                    segment_length, seg_data = validate_seg_dict(
+                        parsed_data[i], resp)
+                    data_for_storage[i] = seg_data
+                    total_size += segment_length
+
         if problem_segments:
             resp_body = get_response_body(
                 out_content_type, {}, problem_segments)
             raise HTTPBadRequest(resp_body, content_type=out_content_type)
         env = req.environ
 
+        slo_etag = md5()
+        for seg_data in data_for_storage:
+            if seg_data.get('range'):
+                slo_etag.update('%s:%s;' % (seg_data['hash'],
+                                            seg_data['range']))
+            else:
+                slo_etag.update(seg_data['hash'])
+
         if not env.get('CONTENT_TYPE'):
             guessed_type, _junk = mimetypes.guess_type(req.path_info)
             env['CONTENT_TYPE'] = guessed_type or 'application/octet-stream'
@@ -1041,6 +1068,9 @@ class StaticLargeObject(object):
         """
         WSGI entry point
         """
+        if env.get('swift.slo_override'):
+            return self.app(env, start_response)
+
         req = Request(env)
         try:
             vrs, account, container, obj = req.split_path(4, 4, True)
