@@ -18,7 +18,6 @@ import os
 import sys
 import time
 import signal
-import re
 from os.path import basename, dirname, join
 from random import shuffle
 from swift import gettext_ as _
@@ -30,13 +29,10 @@ from swift.common.utils import (
     get_logger, ratelimit_sleep, dump_recon_cache, list_from_csv, listdir,
     unlink_paths_older_than, readconf, config_auto_int_value)
 from swift.common.exceptions import DiskFileQuarantined, DiskFileNotExist,\
-    DiskFileDeleted
+    DiskFileDeleted, DiskFileExpired
 from swift.common.daemon import Daemon
 from swift.common.storage_policy import POLICIES
 
-# This matches rsync tempfiles, like ".<timestamp>.data.Xy095a"
-RE_RSYNC_TEMPFILE = re.compile(r'^\..*\.([a-zA-Z0-9_]){6}$')
-
 
 class AuditorWorker(object):
     """Walk through file system to audit objects"""
@@ -265,6 +261,8 @@ class AuditorWorker(object):
             self.logger.error(_('ERROR Object %(obj)s failed audit and was'
                                 ' quarantined: %(err)s'),
                               {'obj': location, 'err': err})
+        except DiskFileExpired:
+            pass  # ignore expired objects
         except DiskFileDeleted:
             # If there is a reclaimable tombstone, we'll invalidate the hash
             # to trigger the replicator to rehash/cleanup this suffix
@@ -279,8 +277,8 @@ class AuditorWorker(object):
         # _ondisk_info attr is initialized to None and filled in by open
         ondisk_info_dict = df._ondisk_info or {}
         if 'unexpected' in ondisk_info_dict:
-            is_rsync_tempfile = lambda fpath: RE_RSYNC_TEMPFILE.match(
-                basename(fpath))
+            is_rsync_tempfile = lambda fpath: (
+                diskfile.RE_RSYNC_TEMPFILE.match(basename(fpath)))
             rsync_tempfile_paths = filter(is_rsync_tempfile,
                                           ondisk_info_dict['unexpected'])
             mtime = time.time() - self.rsync_tempfile_timeout
